<?php

namespace Drupal\media_entity_calameo\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media_entity_calameo\Plugin\media\Source\Calameo;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'calameo_embed' formatter.
 *
 * @FieldFormatter(
 *   id = "calameo_embed",
 *   label = @Translation("Calameo embed"),
 *   field_types = {
 *     "link", "string", "string_long"
 *   }
 * )
 */
class CalameoEmbedFormatter extends FormatterBase {

  /**
   * Calameo reading url.
   */
  const CALAMEO_URL = '//fr.calameo.com/read/';

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    /** @var \Drupal\media\MediaInterface $media */
    $media = $items->getEntity();
    $settings = $this->getSettings();

    if (empty($media)
      || $media instanceof Calameo) {
      return [];
    }

    if (!$media->hasField('field_media_calameo')
    || !$calameo_id = $media->field_media_calameo->value) {
      return [];
    }

    if (stripos($calameo_id, 'http') !== FALSE) {
      $iframe_url = Url::fromUri($calameo_id);
    } else {
      $iframe_url = Url::fromUri(self::CALAMEO_URL . $calameo_id);
    }

    return [
      '#theme' => 'media_calameo',
      '#url' => $iframe_url->toString(),
      '#width' => $settings['width'],
      '#height' => $settings['height'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return $field_definition->getTargetEntityTypeId() === 'media';
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'width' => '480',
        'height' => '640',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#default_value' => $this->getSetting('width'),
      '#min' => 1,
      '#description' => $this->t('Width of Calameo.'),
    ];

    $elements['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#default_value' => $this->getSetting('height'),
      '#min' => 1,
      '#description' => $this->t('Height of Calameo.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    return [
      $this->t('Width: @width px', [
        '@width' => $this->getSetting('width'),
      ]),
      $this->t('Height: @height px', [
        '@height' => $this->getSetting('height'),
      ]),
    ];
  }

}
